/*
   Simple Graphics Library - SGL kniznica pre PMD 85-2A/3 a BDS C kompilator.
   Subor kniznice: SGL.CRL
   Verzia: 1.0
   Autor: RomBor 09-11/2021

   Hlavickovy subor pre SGL.CRL
*/

/*
   Konstanty pre volbu pociatku suradnice Y.
   Pozuva sa jedna z nich pri volani funkcie gOriginY().
 */

/* Suradnica Y ma pociatok hore a jej hodnota stupa zhora dole od 0 do 255. */
#define ORG_Y_TOP	0
/* Suradnica Y ma pociatok dole a jej hodnota klesa zhora dole od 255 do 0. */
#define ORG_Y_BOTTOM	1
/* Suradnica Y ma pociatok v strede a jej hodnota klesa zhora dole od 127 do -128. */
#define ORG_Y_MIDDLE	2

/*
   Konstanty pre volbu rezimu vykreslovania bodov, ciar a textu.
   Pouzivaju sa pri volani funkcie gMode(), ako ich kombinacia binarnym operatorom `|`.
   Kod farby SGL_COLOR_* ma vo vyraze zmysel iba jeden.
   Konstanty SGL_MODE_* sa mozu navzajom kombinovat, ale nie vsetky varianty maju zmysel.
   Konstanta SGL_MODE_NOT sa uplatnuje iba pri vykreslovani bodov a ciar.
   Konstanty SGL_DOUBLE_* sa uplatnuju iba pri vypise znakov funkciou gPrint().
*/

/* kody farby */
#define COLOR_0		0
#define COLOR_1		1
#define COLOR_2		2
#define COLOR_3		3

/* maska pre inverzne vykreslovanie */
#define MODE_INV	4
/* maska pre vykreslovanie nastavovanim */
#define MODE_SET	0
/* maska pre vykreslovanie negovanim */
#define MODE_NEG	8
/* maska pre vykreslovanie mazanim */
#define MODE_NOT	0x10

/* maska pre volbu dvojitej vysky znakov */
#define DBL_HEIGHT	0x40
/* maska pre volbu dvojitej sirky znakov */
#define DBL_WIDTH	0x80

/*
   ----------------------------------------------------------------------------
   * Zoznam vsetkych funkcii kniznice. *
   ----------------------------------------------------------------------------

   ----------------------------------------------------------------------------
   * gInit *
   Inicializacia grafickej kniznice.
   Funkcia musi byt zavolana skor, nez bude zavolana ina funkcia kniznice.
   Navratova hodnota je =0 pre PMD 85-2A a <>0 pre PMD 85-3.

	int gInit()

   ----------------------------------------------------------------------------
   * gOriginY *
   Nastavenie pociatku suradnice Y.
   Nastaveny pociatok sa vztahuje na vykreslovanie bodov, ciar i textu.
   Parameter O je jedna z konstant SGL_ORIGIN_Y_*. Ine hodnoty sa ignoruju.

	void gOriginY(O)
	int O;

   ----------------------------------------------------------------------------
   * gMode *
   Nastavenie rezimu vykreslovania bodov, ciar a textu.
   Parameter M sa vyskladava z vyssie uvedenych konstant SGL_COLOR_*, SGL_MODE_*
   a SGL_DOUBLE_*.

	void gMode(M)
	int M;

   ----------------------------------------------------------------------------
   * gCls *
   Zmazanie obrazovky.
   Pri mazani sa vyuziva nastaveny rezim gMode(), teda zvolena farba SGL_COLOR_*
   a inverzia SGL_MODE_INV.

	void gCls()

   ----------------------------------------------------------------------------
   * gPrintPos *
   Nastavenie pozicie vypisu textu.
   Z hodnoty X a Y sa berie iba nizsi byte (vyssi byte sa ignoruje).
   Suradnica X musi byt v intervale <0, 47>, teda ide o znakovu poziciu. Ak je
   mimo tohto intervalu, pouzije sa hodnota 0.
   Suradnica Y sa vyhodnocuje podla nastavenia funkciou gOriginY() a je v intervale
   SGL_ORIGIN_Y_TOP: 	<0, 255>
   SGL_ORIGIN_Y_BOTTOM:	<255, 0>
   SGL_ORIGIN_Y_MIDDLE:	<127, -128>
   Suradnica Y sa udava tak, ze sa urcuje pozicia pre lavy dolny roh znaku a znak
   sa vypisuje zdola hore.

	void gPrintPos(X, Y)
	int X, Y;

   ----------------------------------------------------------------------------
   * gPrint *
   Vypisanie retazca urceneho agumentom pT (pointer na char - adresa retzca) na
   aktualnu poziciu, resp. na poziciu urcenu funkciou gPrintPos().
   Na vykreslenie sa pouzije rezim nastaveny funkciou gMode().
   Znaky sa vypisuju zdola hore. Ak pri vypisovani znak "opusti" obrazovku, nebude
   vypisany cely. Ak sa ale dosiahne pravy okraj obrazovky a retazec dalej pokracuje,
   vypisovanie bude pokracovat na dalsom riadku, kde je zaroven zohladnena aktualna
   vyska znaku. Pri jednoduchej vyske znaku ma riadok 10 mikroriadkov, pri dvojitej
   vyske ma radok 18 mikroriadkov.
   V retazci znakov mozno pouzit aj riadiaci znak '\n' (kod 0x0A) na okamzity prechod
   na novy riadok, opat s ohladom na aktualne nastavenu vysku znakov. Ostatne
   riadiace znaky sa nezohladnuju a zobrazia sa ako "nedefinovany" stvorcek.

	void gPrint(pT)
	char *pT;

   ----------------------------------------------------------------------------
   * gMove *
   Presunutie grafickeho kurzora na suradnice X a Y.
   Z hodnoty X a Y sa berie iba nizsi byte (vyssi byte sa ignoruje).
   Suradnica X je tada v intervale <0, 255>.
   Suradnica Y sa vyhodnocuje podla nastavenia funkciou gOriginY() a je v intervale
   SGL_ORIGIN_Y_TOP: 	<0, 255>
   SGL_ORIGIN_Y_BOTTOM:	<255, 0>
   SGL_ORIGIN_Y_MIDDLE:	<127, -128>

	void gMove(X, Y)
	int X, Y;

   ----------------------------------------------------------------------------
   * gPlot *
   Presunutie grafickeho kurzora na suradnice X a Y a vykreslenie bodu.
   Na vykreslenie sa pouzije rezim nastaveny funkciou gMode().
   Z hodnoty X a Y sa berie iba nizsi byte (vyssi byte sa ignoruje).
   Suradnica X je tada v intervale <0, 255>.
   Suradnica Y sa vyhodnocuje podla nastavenia funkciou gOriginY() a je v intervale
   SGL_ORIGIN_Y_TOP: 	<0, 255>
   SGL_ORIGIN_Y_BOTTOM:	<255, 0>
   SGL_ORIGIN_Y_MIDDLE:	<127, -128>

	void gPlot(X, Y)
	int X, Y;

   ----------------------------------------------------------------------------
   * gDraw *
   Vykreslenie ciary od posledneho bodu nastaveneho gMove(), gPlot() alebo
   gDraw() do bodu zadaneho suradnicami X a Y.
   Na vykreslenie sa pouzije rezim nastaveny funkciou gMode().
   Z hodnoty X a Y sa berie iba nizsi byte (vyssi byte sa ignoruje).
   Suradnica X je tada v intervale <0, 255>.
   Suradnica Y sa vyhodnocuje podla nastavenia funkciou gOriginY() a je v intervale
   SGL_ORIGIN_Y_TOP: 	<0, 255>
   SGL_ORIGIN_Y_BOTTOM:	<255, 0>
   SGL_ORIGIN_Y_MIDDLE:	<127, -128>

	void gDraw(X, Y)
	int X, Y;

   ----------------------------------------------------------------------------
   * gSin *
   Vypocet hodnoty goniometrickej funkcie Sinus.
   Vzhladom na celociselnu povahu BDS C kompilatora, je aj "vypocet" prisposobeny
   tejto skutocnosti a zaroven je viazany na graficke rozlisenie PMD 85.
   Argument funkcie je v intervale <0,255>, co zodpoveda beznemu intervalu <0,2PI>.
   Interval funkcnej hodnoty zodpoveda nastaveniu pociatku suradnice Y funkciou
   gOriginY().
   Z hodnoty argumentu V sa berie iba nizsi byte (vyssi byte sa ignoruje).

	int gSin(V)
	int V;

   ----------------------------------------------------------------------------
   * gCos *
   Vypocet hodnoty goniometrickej funkcie Cosinus.
   Pre funkciu gCos() plati to iste ako pre funkciu gSin().

	int gCos(V)
	int V;

   ----------------------------------------------------------------------------
   * gSinDA * (Sinus / Div / Add)
   Vypocet hodnoty goniometrickej funkcie Sinus, ktora je vydelena hodnotou D
   a je ku nej pripocitana hodnota A. Vykona sa teda tento vypocet:
	S = gSin(V) / D + A;
   Tato funkcia bola pridana preto, ze je to, vzhladom na celocisenu povahu
   Sin/Cos a cielove rozlisenie, casto pouzivanym vypoctom a priamy vypocet
   v ramci samotnej funkcie je rychlejsi, ako vykonanie jednotlivych operacii
   samostatne. Hodnoty D a A mozu byt aj zaporne.

	int gSinDA(V, D, A)
	int V, D, A;

   ----------------------------------------------------------------------------
   * gCosDA * (Cosinus / Div / Add)
   Vypocet hodnoty goniometrickej funkcie Cosinus, ktora je vydelena hodnotou D
   a je ku nej pripocitana hodnota A.
   Plati svetko, co bolo popisane k funkcii gSinDA().

	int gCosDA(V, D, A)
	int V, D, A;

   ----------------------------------------------------------------------------
   * gFill *
   Vyplnenie plochy vzorkou danou parametrom P od pozicie zadanej parametrami
   X a Y, pre ktore plati to iste, ako pri funkcii gPrintPos(). Suradnica Y ale
   urcuje lavy horny roh plochy, takze sa vyplnuje zhora dole.
   Sirka a vyska vyplne sa zadava parametrami W a H, z ktorych sa opat berie iba
   nizsi byte. Sirka je v znakovych poziciach. Vyska v mikroriadkoch.
   Vyssi a nizsi byte parametra P predstavuju dve vzorky, ktore sa aplikuju na
   dvojice mikroriadkov. Z oboch bytov sa berie do uvahy iba dolnych 6 bitov.
   Na vyplnenie sa pouzije rezim nastaveny funkciou gMode().
   Ak vypln svojou velkostou presahuje cez pravy alebo dolny okraj obrazovky,
   nevykresli sa cela.

	void gFill(X, Y, W, H, P)
	int X, Y, W, H, P;

   ----------------------------------------------------------------------------
   * gSprite *
   Vykreslenie spritu modom M na poziciu zadanu parametrami X, Y, pre ktore
   plati to iste, ako pri funkcii gPrintPos(). Suradnica Y ale urcuje lavy horny
   roh spritu, takze sprite sa vykresluje zhora dole.
   Sirka a vyska spritu sa zadava parametrami W a H, z ktorych sa opat berie iba
   nizsi byte. Sirka je v znakovych poziciach. Vyska v mikroriadkoch.
   Mod M moze byt MODE_SET alebo MODE_NEG, ine nemaju vyznam.
   Data spritu sa predavaju parametrom pData, co je pointer na char, teda adresa
   pamati, kde su data spritu ulozene.
   Ak sprite svojou velkostou presahuje cez pravy alebo dolny okraj obrazovky,
   nevykresli sa cely.

	void gSprite(X, Y, W, H, M, pData)
	int X, Y, W, H, M;
	char *pData;

   ----------------------------------------------------------------------------
   * gVramByte *
   Vrati byte z VideoRAM zodpovedajuci pozicii zadanej parametrami X a Y, pre
   ktore plati to iste, ako pri funkcii gPrintPos().

	int gVramByte(X, Y)
	int X, Y;

   ----------------------------------------------------------------------------
   * gVramBlk *
   Precitanie bloku VRAM z pozicie zadanej parametrami X, Y, pre ktore plati to
   iste, ako pri funkcii gPrintPos(). Suradnica Y ale urcuje lavy horny roh bloku.
   Sirka a vyska bloku sa zadava parametrami W a H, z ktorych sa opat berie iba
   nizsi byte. Sirka je v znakovych poziciach. Vyska v mikroriadkoch.
   Data bloku sa ulozia od adresy zadanej parametrom pData, co je pointer na char.
   Velkost pola adresovaneho parametrom pData musi mat velkost aspon W*H bytov.
   Ak blok svojou velkostou presahuje cez pravy alebo dolny okraj obrazovky,
   vratene data mozu byt neplatne.

	void gVramBlk(X, Y, W, H, pData)
	int X, Y, W, H;
	char *pData;

   ----------------------------------------------------------------------------
   * gKey *
   Test stlpca klavesnice. Parameter Col urcuje cislo stlpca klavesnice 0 az 15
   (z hodnoty sa beru do uvahy iba dolne 4 bity). Nasledne sa precita stav daneho
   stlpca a aplikuje sa nan maska z parametra Mask (zmysel ma iba 7 dolnyh bitov).
   Navratova hodnota je teda odmaskovana hodnota precitana z portu klavesnice.
   Funkcia gKey() teda prevedie tieto instrukcie:
							mvi	a,Col
							out	0F4h
							in	0F5h
							ani	Mask

	int gKey(Col, Mask)
	int Col, Mask;

   ----------------------------------------------------------------------------
   * gInkey *
   Test klavesnice s cakanim, alebo bez cakania.
   Ak je parameter Wait=0, klavesnica sa otestuje bez cakania s okamzitym
   navratom.
   Ak je parameter Wait<>0, klavesnica sa otestuje s cakanim na stlacenie nejakeho
   klavesu.
   Navratova hodnota je kod stlaceneho klavesu, alebo 0, ak nebolo nic stlacene a
   (Wait=0).

	int gInkey(Wait)
	int Wait;

   ----------------------------------------------------------------------------
   * gBeep *
   Pipnutie o danej vyske a dlzke. Z oboch hodnot sa beru do uvahy iba nizsie
   byty. Nizsia hodnota parametra Pitch dava vyssi ton.
   Ako specialita moze mat parameter Pitch zapornu hodnotu, kedy sa nevyda
   obycajne pipnutie, ale tzv. pisknutie, kedy sa pociatocna frekvencia postupne
   zvysuje. Opat sa z hodnoty ale berie iba nizsi byte, vyssi byte len urcuje,
   ci je hodnota kladna alebo zaporna.

	void gBeep(Pitch, Dur)
	int Pitch, Dur;

   ----------------------------------------------------------------------------
   * gMelody *
   Prehranie jednohlasej melodie.
   Data melodie sa predavaju parametrom pData, co je pointer na char, teda adresa
   pamati, kde su data melodie ulozene. Jednotlive tony su urcene vzdy dvojicou
   bytov Pitch a Dur, rovnako ako vo funkcii gBeep().
   Pitch hodnota 0FEh predstavuje Pauzu o dlzke Dur.
   Pitch hodnota 0FFh predstavuje znacku opakovania melodie.
   Prehravanie melodie mozno ukoncit klavesom STOP.
  
	void gMelody(pData)
	char *pData;

   ----------------------------------------------------------------------------
*/
