/*
   Demonstracny program ku kniznici SGL pre PMD 85-2A/3 BDS C kompilator.

   Subor: SGLDEMO.H
   Autor: RomBor 09-11/2021
*/

/* ------------------------------------------------------------------------- */
#define DIA_FILE      "DIAFONT.FNT"
#define DIA_LEN       512
#define SPR_FILE      "SPRITES.SPR"
#define SPR_LEN       2176
#define SPR2_FILE     "SPRITES2.SPR"
#define SPR2_LEN      140
#define ROOM_FILE     "ROOM.DAT"
#define ROOM_LEN      600
#define MLDY_FILE     "MELODY.DAT"
#define MLDY_LEN      222

/* ------------------------------------------------------------------------- */
/* Pointer na buffer pre font znakov s diakritikou */
char *pDiaFont;
#define DIA1          (pDiaFont+8)
#define DIA2          (pDiaFont+256+8)
#define DIAADR        0xC0BC

/* Pointer na buffer pre data spritov */
char *pSprites;
#define S_MANIC       (pSprites+0)
#define S_LEECH       (pSprites+256)
#define S_OCTOP       (pSprites+640)
#define S_UFO         (pSprites+1024)
#define S_WORM        (pSprites+1408)
#define S_BIRD        (pSprites+1792)

/* Pointer na buffer grafiky miestnosti */
char *pSprRoom;
#define S_EMPTY       (pSprRoom+0)
#define S_FLOOR       (pSprRoom+10)
#define S_RFLOOR      (pSprRoom+20)
#define S_LFLOOR      (pSprRoom+30)
#define S_XFLOOR      (pSprRoom+40)
#define S_FEED1       (pSprRoom+50)
#define S_FEED2       (pSprRoom+60)
#define S_FEED3       (pSprRoom+70)
#define S_FEED4       (pSprRoom+80)
#define S_KEY         (pSprRoom+90)
#define S_LADDER      (pSprRoom+100)
#define S_LSTEP       (pSprRoom+110)
#define S_RSTEP       (pSprRoom+120)
#define S_ROPE        (pSprRoom+130)

/* Pointer na buffer dat miestnosti */
char *pRoom;
#define ROOM_X        4     /* pociatocny znakovy stlpec */
#define ROOM_Y        100   /* pociatocny mikroriadok z hora (musi byt delitelny 5!) */
#define ROOM_W        20    /* sirka v dielikoch */
#define ROOM_H        30    /* vyska v dielikoch */
#define TILE_W        2     /* sirka dielika v znakovych poziciach */
#define TILE_H        5     /* vyska dielika v mikroriadkoch */
#define SPR_H         16    /* vyska pohyblivych spritov v mikroriadkoch */

/* Pointer na buffer dat melodie */
char *pMelody;

/* ------------------------------------------------------------------------- */
/* konstanty pre stav Manica */
#define MS_IDL        0   /* Manic v klude */
#define MS_JMP        1   /* jednoduchy vyskok */
#define MS_RGH        2   /* Manic v pohybe do prava */
#define MS_LFT        3   /* Manic v pohybe vlavo */
#define MS_RJMP       4   /* vyskok vpravo */
#define MS_LJMP       5   /* vyskok vlavo */
#define MS_FALL       6   /* padanie */

/* konstanty ovladacich klavesov */
#define K_NONE        0
#define K_JMP         1
#define K_RGH         2
#define K_LFT         3
#define K_RJMP        4
#define K_LJMP        5

/* ------------------------------------------------------------------------- */
/* struktura spritu Manic Minera */
struct SPRM {
  char *pManS;    /* adresa aktualneho spritu */
  int mI;         /* index spritu */
  int mX;         /* aktualna pozicia X */
  int mY;         /* pozicia Y */
  int mD;         /* smer spritu 0/1=R/L */
  int mS;         /* stav Manica - konstanty MS_* */
  int mK;         /* naposledy stlaceny klaves */
  int mA;         /* pomocna premenna 1 */
  int mB;         /* pomocna premenna 2 */
  int mC;         /* pomocna premenna 3 */
  int mE;         /* pomocna premenna 4 */
};

/* struktura spritu "obludy" */
struct SPRO {
  char *pOblD;    /* pociatocna adresa dat spritov */
  char *pOblS;    /* adresa aktualneho spritu */
  int oI;         /* index spritu */
  int oX;         /* aktualna pozicia X */
  int oY;         /* pozicia Y */
  int oD;         /* smer spritu 0/1=R/L */
  int oXmin;      /* minimalna pozicia X spritu */
  int oXmax;      /* maximalna pozicia X spritu */
};

/* struktura pohybliveho pasu */
struct SPRF {
  char *pFeedS;   /* adresa aktualneho spritu */
  int fI;         /* index spritu */
  int fX;         /* pozicia X */
  int fY;         /* pozicia Y */
};

/* struktura Manica */
struct SPRM manic;

/* pole struktur spritov Obludy */
#define MAX_SPRO      5
struct SPRO sprites[MAX_SPRO];
int sproCnt;

/* pole struktur posuvnych pasov */
#define MAX_SPRF      10
struct SPRF feeds[MAX_SPRF];
int sprfCnt;

/* ------------------------------------------------------------------------- */
/* pole vysky pipnutia pri skoku Manica */
int beepTab[15];

/* pocet klucov v miestnosti a kolizie */
char keysCnt[3];
/* char clashCnt[6]; */

/* pole pre dieliky okolo Manica */
char around[18];

/* priznak kolizie alebo padu z vysky */
int flgFail;

/* ------------------------------------------------------------------------- */
